@@ -20,16 +20,12 @@
 open Printf
 open Benl_core
 open Benl_base
-open Benl_marshal
-open Html5.M
-
-module M = Package.Map
-module S = Package.Set
+open Tyxml.Html
 
 let use_colors = ref false
 let output_file = ref None
 let input_source = ref Benl_types.NoSource
-let baseurl = ref "file:///.."
+let baseurl = ref "https://people.canonical.com/~ubuntu-archive/transitions/"
 
 type output_format = Text | Xhtml | Levels | Json
 let output_format = ref Levels
@@ -40,11 +36,6 @@ let output_format_from_string = function
   | "color" -> use_colors := true; Text
   | _ -> Levels
 
-let p = Benl_clflags.progress
-let ( // ) = Filename.concat
-let ( !! ) = Lazy.force
-let ( !!! ) = Package.Name.to_string
-
 let map_fun f l =
   let fr = Benl_frontend.get_selected_frontend () in
   if fr.Benl_frontend.name = "monitor" then
@@ -62,11 +53,6 @@ let is_good config =
   lazy (Query.of_expr (get_config config "is_good"))
 let is_bad config =
   lazy (Query.of_expr (get_config config "is_bad"))
-let to_forget = List.fold_left
-  (fun accu x -> Benl_base.Fields.add x accu) Benl_base.Fields.empty
-  [
-    "description";
-  ]
 
 open Benl_modules
 open Marshallable
@@ -89,15 +75,13 @@ let format_arch x =
     else f
   in f
 
-let ben_webpage = "http://ben.debian.net"
+let ben_webpage = "https://ben.debian.net"
 
 let print_dep_line src deps =
   printf "%s:" !!!src;
   S.iter (fun dep -> printf " %s" !!!dep) deps;
   printf "\n%!"
 
-let print_dep_graph x = M.iter print_dep_line x
-
 let spec = Arg.align [
   "--run-debcheck", Arg.Set Benl_data.run_debcheck, " Run debcheck to register installability information";
   "--use-projectb", Arg.Set Benl_data.use_projectb, " Get package lists from Projectb database";
@@ -140,9 +124,6 @@ let check_media_dir base =
         end
     | _ -> ()
 
-let relevant_arch arch_ref arch_pkg =
-  (arch_pkg = "all" && arch_ref = "i386") || arch_ref = arch_pkg
-
 let compute_state config pkg =
   if Query.eval_binary pkg !!(is_bad config) then
     if Query.eval_binary pkg !!(is_good config) then
@@ -230,9 +211,9 @@ let print_text_monitor config sources binaries rounds =
     let left = width/2 in
     let right = width-left in
     let buf = Buffer.create 64 in
-    for i = 1 to left do Buffer.add_char buf '=' done;
+    for _ = 1 to left do Buffer.add_char buf '=' done;
     bprintf buf "> Dependency level %%%dd <" nrounds;
-    for i = 1 to right do Buffer.add_char buf '=' done;
+    for _ = 1 to right do Buffer.add_char buf '=' done;
     Buffer.add_char buf '\n';
     Scanf.format_from_string (Buffer.contents buf) "%d"
   in
@@ -279,36 +260,33 @@ let a_link url text =
 let escape = Netencoding.Url.encode
 
 let pts t src =
-  a_link (t.Template.pts (escape src)) src
+  a_link (t.Template.pts ~src:(escape src)) src
 
-let buildd t show src ver =
-  if show
-  then a_link (t.Template.buildd src ver) "build logs"
-  else small [ pcdata "arch:all" ]
+let buildd t src ver =
+  a_link (t.Template.buildd ~src ~ver) "build logs"
 
-let buildds t show srcs =
-  match t.Template.buildds srcs with
+let buildds t srcs =
+  match t.Template.buildds ~srcs with
   | None -> raise Exit
   | Some s -> a_link s "build logs"
 
 let rc_bugs t src =
-  match t.Template.critical_bugs src with
+  match t.Template.critical_bugs ~srcs:src with
   | None -> raise Exit
   | Some s -> a_link s "RC bugs"
 
 let changelog t ver dir src =
-  small [ a_link ( t.Template.changelog dir src ver) ver ]
+  small [ a_link ( t.Template.changelog ~letter:dir ~src ~ver) ver ]
 
 let generated_on_text () =
   [ pcdata "Page generated by ";
-    a_link ben_webpage "Ben";
+    a_link ben_webpage "Ben ";
+    pcdata Benl_version.version;
     pcdata (Printf.sprintf " on %s" (Benl_core.get_rfc2822_date ()));
     pcdata "; bzr branch ";
     a_link "https://code.launchpad.net/+branch/ubuntu-transition-tracker" "lp:ubuntu-transition-tracker"
   ]
 
-module SS = Set.Make(String)
-
 let overrall_state l =
   let _ (* ignored_archs *), release_archs =
     List.partition
@@ -375,22 +353,22 @@ let beautify_text =
       (function
         | Pcre.Text s -> pcdata s
         | Pcre.Delim s ->
-          let l = String.lowercase s in
+          let l = String.lowercase_ascii s in
           if s.[0] = '#' then
             let ss = String.sub s 1 (String.length s -1) in
-            let link = tpl.Template.bugs ss in
+            let link = tpl.Template.bugs ~src:ss in
             a_link link s
           else if s.[0] = '<' then
             let ss = String.sub s 1 (String.length s - 2) in
-            let link = tpl.Template.msg_id ss in
+            let link = tpl.Template.msg_id ~mid:ss in
             a_link link s
           else if starts_with l "pts" then
             let text = cut_head s "pts:" in
-            let link = tpl.Template.pts text in
+            let link = tpl.Template.pts ~src:text in
             a_link link s
           else if starts_with l "buildd" then
             let text = cut_head s "buildd:" in
-            let link = tpl.Template.buildd text "" in
+            let link = tpl.Template.buildd ~src:text ~ver:"" in
             a_link link s
           else
             a_link s s
@@ -399,12 +377,7 @@ let beautify_text =
       t
 
 let compute_graph data config =
-  let architectures =
-    List.map
-      (Benl_frontend.to_string "architectures")
-      (archs_list config)
-  in
-  let {src_map = sources; bin_map = binaries} = get_data ~cache:(Some data) is_affected architectures config in
+  let {src_map = sources; bin_map = binaries} = filter_affected data is_affected config in
   let src_of_bin : ([`binary], [`source] Package.Name.t) M.t =
     PAMap.fold
       (fun (name, _) pkg accu ->
@@ -432,7 +405,7 @@ let has_testing_data monitor_data =
      with Not_found -> false)
   | _ -> false
 
-let print_html_monitor config template monitor_data sources binaries dep_graph packages has_testing_data extra =
+let print_html_monitor config template monitor_data binaries dep_graph packages has_testing_data extra =
   let affected = packages in
   let mytitle =
     try
@@ -486,9 +459,11 @@ let print_html_monitor config template monitor_data sources binaries dep_graph p
         pcdata "Filter by status: ";
         input ~a:[a_input_type `Checkbox; a_id "good"] ();
         pcdata "good ";
-        input ~a:[a_input_type `Checkbox; a_checked `Checked; a_id "bad"] ();
+        input ~a:[a_input_type `Checkbox; a_checked (); a_id "partial"] ();
+        pcdata "partial ";
+        input ~a:[a_input_type `Checkbox; a_checked (); a_id "bad"] ();
         pcdata "bad ";
-        input ~a:[a_input_type `Checkbox; a_checked `Checked; a_id "unknown"] (); pcdata "unknown";
+        input ~a:[a_input_type `Checkbox; a_checked (); a_id "unknown"] (); pcdata "unknown";
         span ~a:[a_id "count"] [];
       ];
     div (if has_testing_data
@@ -508,6 +483,8 @@ let print_html_monitor config template monitor_data sources binaries dep_graph p
     | "powerpc" -> "ppc"
     | x -> x in
   let archs_columns = List.map begin function
+    | Benl_types.EString arch when (List.mem arch !Benl_base.ignored_architectures) ->
+      th [ small [ i [ pcdata (abrege arch) ] ] ]
     | Benl_types.EString arch ->
       th [ small [ pcdata (abrege arch) ] ]
     | _ -> assert false
@@ -518,8 +495,9 @@ let print_html_monitor config template monitor_data sources binaries dep_graph p
   let rows, _ =
     List.fold_left begin fun (rows, i) xs ->
       let names, rows =
-      (List.fold_left begin fun (arch_any_s, acc) (source, states) ->
+      (List.fold_left begin fun (sources, acc) (source, states) ->
         let src = Package.get "package" source in
+        let is_arch_all = Package.get "architecture" source = "all" in
         let has_ma_same =
           List.exists (fun bin ->
             List.exists (fun arch ->
@@ -530,10 +508,22 @@ let print_html_monitor config template monitor_data sources binaries dep_graph p
             ) (List.map (Benl_frontend.to_string "architectures") (archs_list config))
           ) (Package.binaries source)
         in
-        let has_ma_same_html =
-          if has_ma_same then [
+        let has_additional_text, additional_texts =
+          List.fold_left
+            (fun (has, acc) (property, text) ->
+              if property then
+                property || has, text::acc
+              else
+                has, acc
+            )
+            (false, [])
+            [(is_arch_all, "arch:all"); (has_ma_same, "ma:same")]
+        in
+        let additional_text = String.concat ", " additional_texts in
+        let additional_text_html =
+          if has_additional_text then [
             pcdata " [";
-            small [pcdata "ma:same"];
+            small [pcdata additional_text];
             pcdata "]";
           ] else []
         in
@@ -556,8 +546,7 @@ let print_html_monitor config template monitor_data sources binaries dep_graph p
         in
         let version = Package.get "version" source in
         let directory = Package.get "directory" source in
-        let arch_any = Package.get "architecture" source <> "all" in
-        let arch_any_s = if arch_any then src::arch_any_s else arch_any_s in
+        let sources = src::sources in
         let overrall_state = [ class_of_status (overrall_state states) ] in
         let src_text =
           (pts template src)::(if in_testing then [] else [pcdata " (sid only)"])
@@ -570,27 +559,25 @@ let print_html_monitor config template monitor_data sources binaries dep_graph p
                    src_text;
                 td ~a:[ a_class ["src"] ]
                    ([ pcdata "[";
-                     buildd template arch_any (escape src) version;
+                     buildd template (escape src) version;
                      pcdata "] (";
                      changelog template (sprintf "%s" version) directory (escape src);
-                     pcdata ")" ] @ has_ma_same_html)
+                     pcdata ")" ] @ additional_text_html)
               ]
               @
-                (List.map begin fun (arch, (reasons, state)) ->
+                (List.map begin fun (_, (reasons, state)) ->
                                 (td ~a:[ a_class [ Benl_base.class_of_status state ]; a_title reasons ]
                                     [ small [ pcdata (Benl_base.string_of_status state) ] ])
                           end states)
              )
              :: acc
         in
-        arch_any_s, acc
+        sources, acc
       end ([], rows) (List.rev xs)) in
+      let escaped_names = List.map (fun x -> escape x) names in
       let column_arg = try
-        let buildd_link =
-          if names = []
-          then small [ pcdata "arch:all" ]
-          else buildds template true (List.map escape names) in
-        let rc_bugs_link = rc_bugs template (List.map escape names) in
+        let buildd_link = buildds template escaped_names in
+        let rc_bugs_link = rc_bugs template escaped_names in
         [
           pcdata " (";
           buildd_link; pcdata " "; rc_bugs_link;
@@ -619,11 +606,11 @@ let print_html_monitor config template monitor_data sources binaries dep_graph p
      pcdata (Printf.sprintf " → %s" mytitle)
     ] in
   let html = template.Template.page
-    page_title
-    subtitle
-    extra_headers
-    (hbody content)
-    footer
+    ~title:page_title
+    ~subtitle
+    ~headers:extra_headers
+    ~body:(hbody content)
+    ~footer
   in
   html
 
@@ -663,7 +650,6 @@ let main _ =
           config
           template
           monitor_data
-          sources
           binaries
           dep_graph
           packages
@@ -672,7 +658,7 @@ let main _ =
       in
       match !output_file with
       | None ->
-          Html5.P.print print_string output;
+          Format.printf "%a" (Tyxml.Html.pp ()) output;
           print_newline ()
       | Some file ->
           check_media_dir (Filename.basename file);
