@@ -17,7 +17,7 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
-open Html5.M
+open Tyxml.Html
 open Printf
 open Benl_core
 open Benl_clflags
@@ -35,8 +35,15 @@ let tconfig = ref None
 let redownload = ref true
 
 open Benl_types
+open Benl_modules
 open Benl_frontend
 
+let eval x =
+  Query.eval
+    `source
+    (Package.of_assoc `source Benl_core.StringMap.empty)
+    (Query.of_expr x)
+
 let read_global_config () =
   if Sys.file_exists !global_config then begin
     let config = Benl_utils.parse_config_file !global_config in
@@ -63,12 +70,12 @@ let read_global_config () =
         | "mirror", (EString mirror) ->
           Benl_clflags.mirror_sources := mirror;
           Benl_clflags.mirror_binaries := mirror
-        | "use-cache", Etrue ->
-          Benl_clflags.use_cache := true
-        | "run-debcheck", Etrue ->
-          Benl_data.run_debcheck := true
-        | "use-projectb", Etrue ->
-          Benl_data.use_projectb := true
+        | "use-cache", expr ->
+          Benl_clflags.use_cache := eval expr
+        | "run-debcheck", expr ->
+          Benl_data.run_debcheck := eval expr
+        | "use-projectb", expr ->
+          Benl_data.use_projectb := eval expr
         | "base-url", (EString url) ->
           Ben_monitor.baseurl := url
         | "template", (EString template) ->
@@ -100,8 +107,6 @@ let spec = Arg.align [
   "--no-clean"   , Arg.Clear clean, " Do not clean HTML directory before exiting";
 ]
 
-exception Unknown_profile of string
-
 type profile = Planned | Ongoing | Permanent | Finished | Old | Unknown
 
 let string_of_profile = function
@@ -178,8 +183,7 @@ let read_transition_config file =
 let get_transition_data data name config =
   try
     let transition_data = Ben_monitor.compute_transition_data data config in
-    let monitor_data, sources, binaries, dep_graph, all, bad, packages =
-      transition_data in
+    let monitor_data, _, _, _, _, _, _ = transition_data in
     let has_testing_data = Ben_monitor.has_testing_data monitor_data in
     let export =
       try
@@ -242,9 +246,7 @@ let print_html_collisions (hits : (SMap.key * SMap.key list) list) =
   ]
 
 let print_html_monitor config template file transition_data has_testing_data collisions =
-  let monitor_data, sources, binaries, dep_graph, _, _, packages =
-    transition_data
-  in
+  let monitor_data, _, binaries, dep_graph, _, _, packages = transition_data in
   let ($) = Filename.concat in
   let (!!) = Filename.basename in
   let collisions_div =
@@ -259,7 +261,6 @@ let print_html_monitor config template file transition_data has_testing_data col
       config
       template
       monitor_data
-      sources
       binaries
       dep_graph
       packages
@@ -278,7 +279,7 @@ let compute_collisions results =
   let data_map = List.fold_left
     (fun
       data_map
-      (_, transition, _, (_, transition_data, has_testing_data)) ->
+      (_, transition, _, (_, transition_data, _)) ->
         let  _, _, _, _, _, _, pkgs = transition_data in
         let new_data = Benl_data.S.fold
           (fun package data ->
@@ -438,7 +439,7 @@ let tracker template profiles =
   let contents = template.Template.intro @ contents in
   let index = Filename.concat !base "index.html" in
   let subtitle = [ pcdata "Transition tracker" ] in
-  let output = template.Template.page page_title subtitle [] contents footer in
+  let output = template.Template.page ~title:page_title ~subtitle ~headers:[] ~body:contents ~footer in
   try
     p "Generating index...\n";
     dump_xhtml_to_file index output
@@ -451,7 +452,7 @@ let () = at_exit (fun () ->
   with _ -> ()
 )
 
-let main args =
+let main () =
   let () = read_global_config () in
   let lockf = lockf () in
   if test Exists lockf then
@@ -515,10 +516,25 @@ let main args =
       in
       (* Read ben.cache *)
       let cache = read_cache (Benl_base.archs_list ()) in
+      (* Emit warnings for missing source packages *)
+      let () =
+        if !Benl_clflags.verbose || not !Benl_clflags.quiet then
+          let { Marshallable.src_map = srcs; Marshallable.bin_map = bins } = cache in
+          let missing = Hashtbl.create 43 in
+          PAMap.iter begin fun (_, arch) pkg ->
+            let src_name = Package.get "source" pkg in
+            let src = Package.Name.of_string src_name in
+            if not (Package.Map.mem src srcs || Hashtbl.mem missing (src, arch)) then
+              let () = Hashtbl.add missing (src, arch) () in
+              eprintf "warning: Source package %s is missing on %s!\n%!"
+                src_name
+                arch
+          end bins
+      in
       (* Compute data for each transition *)
       let results =
         Benl_parallel.map
-          (fun (name, (config, file, profile)) ->
+          (fun (name, (config, _, profile)) ->
             let () =
               p "Computing data for (%s) %s\n"
                 (string_of_profile profile)
@@ -555,7 +571,7 @@ let main args =
       (* Generate the packages.yaml file *)
       let results =
         Benl_parallel.map
-          (fun (config, t, p, (export, transition_data, has_testing_data)) ->
+          (fun (_, t, p, (export, transition_data, has_testing_data)) ->
             p, t, export, transition_data, has_testing_data
           )
           results
